define([
    "dojo/_base/declare",
    "thornberry/app/Component/_ViewMixin",
    "dojo/_base/lang",
    "thornberry/serverRequest",
    "thornberry/app/utils",
    "dojo/store/Memory",
    "dojo/Deferred",
    "dojo/promise/all",
    "dojo/aspect",
    "dojo/when"
],function(
    declare,
    _ViewMixin,
    lang,
    serverRequest,
    utils,
    Memory,
    Deferred,
    all,
    aspect,
    when
) {
    // Purpose: a view that contains an editor. Helps with things like locking, save validation, and open edit checks/user messaging

    return declare(_ViewMixin,{
        EditorClass: null, // Class extending _EditorComponent if automatic instantiation is desired
        editor: null, // Reference to the instantiated editor

        lockCheckEnabled: false, // Whether to acquire a lock on an object before allowing edits. If true, objectType must be defined
        objectType: "", 
        lockCallClass: "UI.Utility",
        lockCallMethod: "LockObject",

        editFromView: false, 

        _lockStore: null,
        _currentLockId: null,

        getTitle: function() {
            return (this.props.mode === "view" ? "View " : (this.props.objectId == "" ? "Add " : "Edit ")) + this.title;
        },

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                mode: "edit", // view or edit,
                objectId: "",
                viewOnly: false
            });
        },

        getPageProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                content: t.domNode,
                actions : (t.props.mode === "view" ? [
                    { label: "Back", onClick: t.props.onCloseView },
                    { label: "Edit", hidden: t.props.viewOnly, onClick: function() { 
                        t.editFromView = true; 
                        t.set("props",{mode:"edit"})} 
                    },
                ] : [
                    { label: "Cancel", onClick: function() { 
                        if (t.editFromView) {
                            when(t.onBeforeClose(),function(close) {
                                if (close) {
                                    t.set("props",{mode:"view"});
                                    t.editFromView = false;
                                }
                            });
                        } else {
                            t.props.onCloseView()
                        }
                    }},
                    { label: "Save", onClick: function() {t.startSave().then(t.props.onCloseView)}, disabled: !t.editor.modified},
                    { label: "Reset", onClick: function() {t.invalidateRendering()}, disabled: !t.editor.modified }
                ]),
                accountSummaryShown: true,
                helpButtonShown: true
            });
        },

        constructor: function() {
            this._lockStore = new Memory();
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            if (t.EditorClass) {
                t.editor = new t.EditorClass();
                t.domNode = t.editor.domNode;
                t.addSubComponent(t.editor);
            }
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

            if (t.editor) {
                t.setDefaultEditorProps(t.editor);
                t.own(aspect.after(t.editor,"_setValidationPromiseAttr",function(promise) {
                    t.setNavigationDisablePromise(promise);
                },true));
            }
        },

        setDefaultEditorProps: function(editor) {
            var t = this;
            editor.set("props",{
                onChangeModified: function() { t.setPageProps(); }
            });
        },

        validateRendering: function(){
            var t = this;
			if (t.invalidRendering){
                if (t.props.viewOnly) {
                    t.props.mode = "view";
                }
                t.inherited(arguments);
			}
		},

        refreshRendering: function() {
            var t = this;
            if (t.isShown && t.lockCheckEnabled && t.objectType !== "" && t.props.objectId) {
                t.onBeforeLock();
                t.wrapLockCheck(t.props.mode === "edit",t.props.objectId);
            }
            t.inherited(arguments);
        },

        onBeforeLock: function() {
            var t = this;
            t.editor.set("props",{
                disabled: true
            });
        },

        onLockFailure: function(deferred) {
            // default implementation - if we were in view mode before switching to edit, then stay in view mode, otherwise pop the view
            var t = this;
            t.props.mode = "view";
            (t.editFromView ? t.editFromView = false : t.props.onCloseView());
            if (deferred) {
                deferred.resolve();
            }
        },
        
        onLockComplete: function(deferred) {
            // default implementation of hook for performing state changes when an object lock has been obtained
            var t = this;
            t.editor.set("props",{
                disabled: t.props.mode === "view"
            });
            if (deferred) {
                t.editor.readyPromise.then(deferred.resolve);
            }
        },

        // to be called from refreshRendering()
        wrapLockCheck: function(isEditing,id) {
            var t = this;
            if (isEditing && id !== "") {
                var deferred = new Deferred();
                t.addReadyPromise(deferred.promise);
                t.lock(id).then(function() {
                    t.onLockComplete(deferred);
                },function(error) {
                    utils.showMessage(error);
                    t.onLockFailure(deferred);
                });
            } else {
                t.unlock(t._currentLockId);
                t.onLockComplete()
            }
        },

        lock: function(id) {
            var t = this;
            if (id !== t._currentLockId) {
                t.unlock(t._currentLockId);
            }
            t._currentLockId = id;
            var lock = t._lockStore.get(id);
            var lockPromise;
            if (!lock) {
                var lockPromise = serverRequest(t.lockCallClass,t.lockCallMethod,{
                    type: t.objectType,
                    id: id
                });
                t.addReadyPromise(lockPromise);
                t._lockStore.put({ id: id, promise: lockPromise, acquired: false });
                lockPromise.then(function() {
                    t._lockStore.get(id).acquired = true;
                },function(error) {
                    t._lockStore.remove(id);
                });
                t.setNavigationDisablePromise(all(t._readyPromiseArray));
            } else {
                lockPromise = lock.promise;
            }
            return lockPromise;
        },

        unlock: function(id) {
            var t = this;
            if (id) {
                t._currentLockId = null;
                var lock = t._lockStore.get(id)
                if (lock && lock.acquired && !lock.unlocking) {
                    serverRequest(t.lockCallClass,t.lockCallMethod,{
                        type: t.objectType,
                        id: id,
                        release: true
                    }).then(function() {
                        t._lockStore.remove(id);
                    });
                    lock.unlocking = true;
                } else if (lock && !lock.unlocking) {
                    lock.promise.then(function() {
                        t.unlock(id);
                    });
                }
            }
        },

        onClose: function() {
            var t = this;
            t.unlock(t._currentLockId);
            t.editFromView = false;
        },

        onBeforeClose: function() {
            var t = this;
            var result = true;
            if (t.editor.get("modified") && t.props.mode === "edit") {
                var deferred = new Deferred();
                utils.showMessage("Save changes before closing?",[
                    { label: "Yes", callback: function() {
                        t.startSave().then(function() {
                            deferred.resolve(true)
                        },function() {
                            deferred.resolve(false)
                     })} },
                    { label: "No", callback: function() {deferred.resolve(true)} },
                    { label: "Cancel", callback: function() {deferred.resolve(false)} }
                ]);
                result = deferred.promise;
            }
            return result;
        },

        startSave: function() {
            var t = this;
            var deferred = new Deferred();
            if (!t.editor.checkValid()) {
                utils.showMessage("Please correct any invalid fields.");
                deferred.reject();
            } else if (!t.editor.checkRequired()) {
                utils.showMessage("Please fill in all required fields.");
                deferred.reject();
            } else {
                var promise = t.save();
                t.setNavigationDisablePromise(promise);
                promise.then(function() {
                    t.editor.onSaveComplete();
                    deferred.resolve();
                },function(error) {
                    utils.showMessage(error);
                    deferred.reject();
                });
            }
            return deferred.promise;
        },

        save: function() {
            // hook for doing the actual saving and any custom checks. Must return a promise that either resolves or rejects
        }
    });
});