define([
    "dojo/_base/declare",
    "thornberry/serverRequest",
    "dojo/store/Memory",
    "dojo/Deferred",
    "dojo/json",
    "dojo/_base/array",
    "dojo/_base/lang"
],function(
    declare,
    serverRequest,
    Memory,
    Deferred,
    json,
    arrayUtils,
    lang
) {
    var wrapResult = function(result) {
        var deferred = new Deferred();
        deferred.resolve(result);
        return deferred.promise;
    }

    var serviceFactory = function(table,method) {
        var masterStore = new Memory();
        var subscriptions = {};
        var subId = 1;
        return {
            subscribe: function(query,onUpdate) {
                var queryString = json.stringify(query);
                if (!subscriptions[queryString]) { subscriptions[queryString] = [];}
                var id = subId++;
                subscriptions[queryString].push({id:id,onUpdate:onUpdate});
                var store = masterStore.get(queryString);
                if (store && store.store) {
                    onUpdate(store.store.data);
                } else {
                    this.getCollection(query);
                }
                return {remove: function() {
                    subscriptions[queryString] = arrayUtils.filter(subscriptions[queryString],function(sub) {return sub.id !== id});
                }};
            },
            getCollection: function(query,refresh) {
                var queryString = json.stringify(query);
                var store = masterStore.get(queryString);
                if (!store || refresh) {
                    var promise = serverRequest(table,method,query).then(function(result) {
                        arrayUtils.forEach(subscriptions[queryString],function(sub) {
                            sub.onUpdate(result);
                        });
                        masterStore.put({id: queryString, store: new Memory({data:result})});
                        return result;
                    });
                    masterStore.put({id: queryString, promise: promise});
                    return promise;
                } else {
                    return store.promise || wrapResult(store.store.data);
                }
            },
            getSingle: function(query,id,refresh) {
                var queryString = json.stringify(query);
                var store = masterStore.get(queryString);
                if (!store || store.promise || refresh) {
                    return ((store && store.promise) || this.getCollection(query,refresh)).then(function() {
                        return masterStore.get(queryString).store.get(id);
                    });
                } else {
                    return wrapResult(store.store.get(id));
                }
            },
            clearCache: function(query) {
                if (query) {
                    var queryString = json.stringify(query);
                    masterStore.remove(queryString);
                } else {
                    masterStore.setData([]);
                }
            }
        }
    }

    var general = serviceFactory("Patient.Insurance.InsuranceService","GetGeneral");
    var policy = serviceFactory("Patient.Insurance.InsuranceService","GetPolicies");
    var authorization = serviceFactory("Patient.Insurance.InsuranceService","GetAuthorizations");
    var policySequence = serviceFactory("Patient.Insurance.InsuranceService","GetPolicySequence");

    var PatientInsuranceService = declare(null,{
        getGeneral: general.getCollection,
        watchGeneral: lang.hitch(general,"subscribe"),
        saveGeneral: function(id,data) {
            data.ndocNum = id;
            return serverRequest("Patient.Insurance.InsuranceService","SaveGeneral",data);
        },

        /* Policies */
        getPolicies: policy.getCollection,
        getPolicy: lang.hitch(policy,"getSingle"),
        watchPolicies: lang.hitch(policy,"subscribe"),
        savePolicy: function(data) {
            return serverRequest("Patient.Insurance.InsuranceService","SavePolicy",data);
        },
        deletePolicies: function(ids) {
            return serverRequest("Patient.Insurance.InsuranceService","DeletePolicies",ids);
        },
        clearPolicyCache: policy.clearCache,

        /* Authorizations */
        getAuthorizations: authorization.getCollection,
        getAuthorization: lang.hitch(authorization,"getSingle"),
        watchAuthorizations: lang.hitch(authorization,"subscribe"),
        saveAuthorization: function(data) {
            return serverRequest("Patient.Insurance.InsuranceService","SaveAuthorization",data);
        },
        deleteAuthorizations: function(ids) {
            return serverRequest("Patient.Insurance.InsuranceService","DeleteAuthorizations",ids);
        },

        /* Policy Sequence */
        getPolicySequences: policySequence.getCollection,
        getPolicySequence: lang.hitch(policySequence,"getSingle"),
        watchPolicySequences: lang.hitch(policySequence,"subscribe"),
        savePolicySequence: function(data) {
            return serverRequest("Patient.Insurance.InsuranceService","SavePolicySequence",data);
        },
        deletePolicySequences: function(ids) {
            return serverRequest("Patient.Insurance.InsuranceService","DeletePolicySequences",ids);
        },
        lockSequenceDate: function(ndocNum,date) {
            return serverRequest("Patient.Insurance.InsuranceService","LockSequenceDate",{ ndocNum: ndocNum, date: date });
        },

        /* Patient Eligibility Request */
        createEligibilityRequest: function(policyIds) {
            return serverRequest("Billing.Eligibility.Service","CreateRequest",{policyIds:policyIds,useInterface:true});
        }
    });
    return new PatientInsuranceService(); // Singleton
});