define(["dojo/string","dojo/_base/array", "dojo/has"], function(string,arrayUtils, has) {
    /*
        Default Attach Points (must not be used as field names)
            - containerNode
            - titleNode
            - formNode

        Template Specification:

        {
            title: "Editor Title", // optional
            noForm: true/false     // optional; useful when IE8 doesn't allow nested forms
            containers: [
                {
                    attachPoint: "containerDiv",
                    sections: [
                        {
                            title: "SectionTitle", // optional
                            hideDivider: true/false,
                            containerAttachPoint: "attachPoint", // optional
                            rows: [
                                orientation: "horizontal", // or vertical to put the labels above the fields
                                containerAttachPoint: "attachPoint", // optional
                                {
                                    field: {
                                        name: "field", // used to locate corresponding property in the objectModel if it exists. Otherwise, the following properties must be specified
                                        displayName: "Field Name",
                                        fieldSize: "", // small, medium, medium-large, large, grand, etc.
                                        labelSize: "", // small, medium, medium-large, large, grand, etc.
                                        hideLabel: true/false,
                                        containerStyle: "",
                                        noField: false
                                    }
                                    ...or ...
                                    fields: [
                                        {
                                            name: "field", // used to locate corresponding property in the objectModel if it exists. Otherwise, the following properties must be specified
                                            displayName: "Field Name",
                                            fieldSize: "", // small, medium, medium-large, large, grand, etc.
                                            labelSize: "", // small, medium, medium-large, large, grand, etc.
                                            hideLabel: true/false,
                                            containerStyle: "",
                                            noField: false
                                        }, ...
                                    ]
                                }, ...
                            ]
                        }, ...
                    ]
                }, ...
            ]
        }
    */

	var generate = function(specification,objectModel) {
        return _generateBaseTemplate(specification,objectModel);
    };

    var _generateBaseTemplate = function(specification,objectModel) {
        var titleString = _generateTitle(specification);

        var containers = specification.containers || [];
        var containerString = arrayUtils.map(containers,function(container) {
            return  _generateContainer(container,objectModel);
        }).join("");

        return string.substitute(
            '<div class="${baseClass}" data-dojo-attach-point="containerNode" style="width:100%;height:100%">'+
                '<div data-dojo-attach-point="formNode" '+ (specification.noForm ? '' : 'data-dojo-type="dijit/form/Form"') + '>'+
                    '${title}'+
                    '${containers}'+
                '</div>'+
            '</div>',
            { baseClass : "${baseClass}", title : titleString, containers : containerString }
        );
    };

    var _generateTitle = function(specification) {
        return ((specification.title || "") !== "" ? 
            '<div class="content-area" data-dojo-attach-point="titleNode">'+
                '<div class="content-area-title">'+
                    '<div class="content-area-title-text">'+specification.title+'</div>'+
                '</div>'+
            '</div>' : "");
    };

    var _generateContainer = function(container,objectModel) {
        var sections = container.sections || [];
        var sectionString = arrayUtils.map(sections,function(section) {
            return  _generateSection(section,objectModel);
        }).join("");

        return string.substitute(
            '<div '+((container.attachPoint || "") !== "" ? 'data-dojo-attach-point="'+container.attachPoint+'"' : '')+'>'+
                '<div class="content-area">'+
                    '${sections}'+
                '</div>'+
            '</div>',
            { sections : sectionString }
        );
    };

    var _generateSection = function(section,objectModel) {
        var titleString = "";
        var containerAttachPointAttr = (section.containerAttachPoint ? (' data-dojo-attach-point="' + section.containerAttachPoint + '"') : '');
        if (section.title || !section.hideDivider) {
            titleString = 
                '<div class="content-area-title" ' + containerAttachPointAttr + '>'+
                    (section.title ? '<div class="content-area-title-text">'+section.title+'</div>' : '')+
                    (section.hideDivider ? '' : '<div class="content-divider"></div>')+
                '</div>';
        }
        
        var rows = section.rows || [];
        var rowString = arrayUtils.map(rows,function(row) {
            return  _generateRow(row,objectModel);
        }).join("");

        return titleString+rowString;
    };

    var _generateRow = function(row,objectModel) {
        var fieldString = '';
		var containerAttachPointAttr = '';
        if (row.field) {
            fieldString = _generateField(row.field,objectModel,row);
        } else {
			containerAttachPointAttr = (row.containerAttachPoint ? (' data-dojo-attach-point="' + row.containerAttachPoint + '"') : '');
            var fields = row.fields || [];
            fieldString = arrayUtils.map(fields,function(field) {
                return  _generateField(field,objectModel,row);
            }).join("");
        }

        return string.substitute(
            '<div class="content-flex-container" '+containerAttachPointAttr+'>'+
                '${fields}'+
            '</div>',
            { fields : fieldString }
        );
    };
    
    var _generateField = function(field,objectModel,row) {
        var fieldData = _getFieldData(field,objectModel);

        var style = " style='"+(fieldData.containerStyle ? fieldData.containerStyle+';' : "");
        var containerClass = "flex-field-container";
        
        if (row.orientation === "vertical") {
            if (has("ie") == 8) {
                containerClass += " vertical-field-align";
            } else {
                style += "flex-flow:column;-ms-flex-positive:0;flex-grow:0";
            }
        }
        style += "'";

        return string.substitute(
            '<div class="'+containerClass+'" data-dojo-attach-point="${name}Container" '+style+'>'+
                (fieldData.hideLabel ? "" :
                    '<div class="field-name field-name-${labelSize}" data-dojo-attach-point="${name}LabelContainer">'+
                        '<label>${label}</label>'+
                    '</div>')+
                '<div class="field-value field-value-${fieldSize}" data-dojo-attach-point="${name}FieldContainer">'+
                    (fieldData.noField ? '' : '<input data-dojo-attach-point="${name}Node">')+
                '</div>'+
            '</div>',
            fieldData
        );
    };

    var _getFieldData = function(field,objectModel) {
        var propertyModel = (objectModel ? objectModel.getPropertyModel(field.name) : null);
        return {
            name: field.name,
            label: field.displayName || (propertyModel && propertyModel.displayName) || "",
            fieldSize: field.fieldSize || "medium",
            labelSize: field.labelSize || "medium",
            hideLabel: field.hideLabel || false,
            containerStyle: field.containerStyle || "",
            noField: (typeof field.noField === "undefined" ? false : field.noField)
        };
    };

    var autoGenerateFilterTemplate = function(report) {
        var template = 
        '<div class="${baseClass}" data-dojo-attach-point="containerNode">'+
            '<div data-dojo-attach-point="formNode" data-dojo-type="dijit/form/Form">'+
                '<div class="content-area">';

        if (report.filterHelp !== "") {
            template = template + '<div class="content-flex-container">'+
                '<div class="flex-field-container">'+
                    '<div class="field-name emphasize">'+
                        '<label>'+escapeText(report.filterHelp)+'</label>'+
                    '</div>'+
                '</div>'+
            '</div>';
        }

        arrayUtils.forEach(report.filterColumns.split(","),function(column) {
            var column = report.getColumn(column);
            if (column && column.isFilterable && column.filterWidget) {
                var filterName = column.displayName;
                if (column.filterName) {
                    filterName = column.filterName;
                }
                var filterHelp = column.filterHelp;
                if (filterHelp === "") {
                    filterHelp = column.help;
                }
                template = template + 
                    '<div class="content-flex-container">'+
                        '<div class="flex-field-container" title="'+escapeText(filterHelp)+'">'+
                            '<div class="field-name field-name-larger">'+
                                '<label>'+escapeText(filterName)+'</label>'+
                            '</div>'+
                            '<div class="field-value field-value-larger">'+
                                '<input data-dojo-attach-point="'+column.name+'Node">'+
                            '</div>'+
                        '</div>'+
                    '</div>';
            }
        });
        
        template = template +  
                '</div>'+   
            '</div>'+
        '</div>';
        return template;
    };

    var autoGenerateSummarizationTemplate = function(report) {
        var template = 
        '<div class="${baseClass}" data-dojo-attach-point="containerNode">'+
            '<div data-dojo-attach-point="formNode" data-dojo-type="dijit/form/Form">'+
                '<div class="content-area">';

        template = template + 
                '<div class="content-flex-container">'+
                    '<div class="flex-field-container">'+
                        '<div class="field-name field-name-larger">'+
                            '<label>Show summary by</label>'+
                        '</div>'+
                        '<div class="field-value field-value-grander">'+
                            '<input data-dojo-attach-point="summarizeByNode">'+
                        '</div>'+
                    '</div>'+
                '</div>'+ 
                '<div class="content-flex-container">'+
                    '<div class="flex-field-container">'+
                        '<div class="field-name field-name-larger">'+
                            '<label>Show aggregate(s)</label>'+
                        '</div>'+
                        '<div class="field-value field-value-grander">'+
                            '<input data-dojo-attach-point="aggregatesNode">'+
                        '</div>'+
                    '</div>'+
                '</div>';
        
        template = template +  
                '</div>'+   
            '</div>'+
        '</div>';
        return template;
    };

    var autoGenerateCrossTabSummarizationTemplate = function(report) {
        var template = 
        '<div class="${baseClass}" data-dojo-attach-point="containerNode">'+
            '<div data-dojo-attach-point="formNode" data-dojo-type="dijit/form/Form">'+
                '<div class="content-area">';

        template = template + 
                '<div class="content-flex-container">'+
                    '<div class="flex-field-container">'+
                        '<div class="field-name field-name-larger">'+
                            '<label>Summarize rows by</label>'+
                        '</div>'+
                        '<div class="field-value field-value-grander">'+
                            '<input data-dojo-attach-point="summaryRowNode">'+
                        '</div>'+
                    '</div>'+
                '</div>'+ 
                '<div class="content-flex-container">'+
                    '<div class="flex-field-container">'+
                        '<div class="field-name field-name-larger">'+
                            '<label>Summarize columns by</label>'+
                        '</div>'+
                        '<div class="field-value field-value-grander">'+
                            '<input data-dojo-attach-point="summaryColumnNode">'+
                        '</div>'+
                    '</div>'+
                '</div>'+ 
                '<div class="content-flex-container">'+
                    '<div class="flex-field-container">'+
                        '<div class="field-name field-name-larger">'+
                            '<label>Show aggregate</label>'+
                        '</div>'+
                        '<div class="field-value field-value-grander">'+
                            '<input data-dojo-attach-point="aggregatesNode">'+
                        '</div>'+
                    '</div>'+
                '</div>';
        
        template = template +  
                '</div>'+   
            '</div>'+
        '</div>';
        return template;
    };
    
	return {
        generate: generate,
        autoGenerateFilterTemplate: autoGenerateFilterTemplate,
        autoGenerateSummarizationTemplate: autoGenerateSummarizationTemplate,
        autoGenerateCrossTabSummarizationTemplate: autoGenerateCrossTabSummarizationTemplate
    };
});