/*
	thornberry/patientStatus/StatusEditor.js
	
	Author: wamesbury
	Created: 2018-07-31
	Purpose: Patient status editor
	History:
	2019-02-08 WRA ND-8431 Added stopNonSkilledCarePlans field for patient discharges.
	2018-12-04 WRA ND-8022 Don't trigger the "update object value when a modification to one field causes another field to change
							from invalid to valid" logic for the field in focus.
	2018-11-09 DJA ND-7996 
	2018-10-11 WRA ND-7754 Corrected issue when settings.employeeActivitySite is false.
	2018-09-27 WRA ND-7580 Corrected discharge.allowDeleteIncompleteOrders setting hiding the deleteIncompleteMedications field.
*/
define([
	"dojo/_base/declare",
	"thornberry/object/ObjectEditor",
	"thornberry/object/ObjectModel",
	"thornberry/object/template",
	"thornberry/serverRequest",
	"thornberry/object/TableManager",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/aspect",
	"dojo/date/locale",
	"dojo/Deferred",
	"dojo/dom-construct",
	"dojo/dom-style",
	"dojo/dom-class",
	"dojo/on",
	"dojo/query",
	"dojo/date",
	"dijit/registry",
	"dijit/form/DropDownButton",
	"dijit/DropDownMenu",
	"dijit/MenuItem",
	"dijit/form/Textarea",
	"thornberry/patientStatus/LevelOfCareWidget",
	"thornberry/patientStatus/LocationOfCareWidget"
], function(declare, ObjectEditor, ObjectModel, template, serverRequest, TableManager, arrayUtils, lang, aspect, locale, Deferred,
			 domConstruct, domStyle, domClass, on, query, date, registry, DropDownButton, DropDownMenu, MenuItem, Textarea, LevelOfCareWidget, LocationOfCareWidget) {

	var numberRegex = "[1-9]([0-9])?";

	var narrativeFields = [
		"rnsSnaNarrative",
		"snDischargePlan",
		"ptDischargePlan",
		"otDischargePlan",
		"stDischargePlan",
		"mswDischargePlan",
		"ntDischargePlan",
		"rtDischargePlan",
		"mchDischargePlan",
		"additionalDischargePlan",
		"dischargeNeeds",
		"dischargePreferences",
		"dischargeGoals",
		"dischargeTransferArrangements",
		"dischargeInstructions",
		"dischargeSummary"
	];

	var dischargePlanFields = [
		"snDischargePlan",
		"ptDischargePlan",
		"otDischargePlan",
		"stDischargePlan",
		"mswDischargePlan",
		"ntDischargePlan",
		"rtDischargePlan",
		"mchDischargePlan",
		"additionalDischargePlan"
	];

	var booleanOptions = { 1: "Yes", 0: "No" };
	var statusProperties = [];
	var addProperty = function(property, displayName, type, tableName, displayIndex) {
		statusProperties.push({ name: property, displayName: displayName, type: type, tableName: tableName, displayIndex: displayIndex });
	};
	
	addProperty("referralStatus", "Referral Status", "text");
	addProperty("referralDate", "Referral Date", "date");

	addProperty("socDate", "SOC Date", "date");
	addProperty("anticipatedFirstVisit", "Anticipated 1st Visit", "date");

	addProperty("rnsDate", "RNS Date", "date");
	addProperty("rnsReason", "RNS Reason", "tableItem", "Tables.RNSSNAReason");

	addProperty("snaDate", "SNA Date", "date");
	addProperty("snaReason", "SNA Reason", "tableItem", "Tables.RNSSNAReason");

	addProperty("rnsSnaNarrative", "RNS/SNA Narrative", "text");

	addProperty("deleteScheduledVisits", "Delete uncharted scheduled visits?", "enum", "", booleanOptions);
	addProperty("deleteScheduledMeetings", "Delete all future meetings scheduled?", "enum", "", booleanOptions);

	addProperty("errorMedications", "Chart all medications as an error?", "enum", "", booleanOptions);
	addProperty("discontinueMedications", "Discontinue all medications?", "enum", "", booleanOptions);
	addProperty("deleteIncompleteMedications", "Delete all incomplete medications?", "enum", "", booleanOptions);
	addProperty("holdActiveMedications", "Place all active medications on hold?", "enum", "", booleanOptions);
	addProperty("resumeMeds", "Resume medications that were placed on Hold when charting a Transfer?", "enum", "", booleanOptions);

	addProperty("errorOrders", "Chart all physician orders as an error?", "enum", "", booleanOptions);
	addProperty("discontinueOrders", "Discontinue all physician orders?", "enum", "", booleanOptions);
	addProperty("deleteIncompleteOrders", "Delete all incomplete physician orders?", "enum", "", booleanOptions);
	addProperty("holdActiveOrders", "Place all active physician orders on hold?", "enum", "", booleanOptions);
	addProperty("resumeOrders", "Resume physician orders that were placed on Hold when charting a Transfer?", "enum", "", booleanOptions);

	addProperty("stopNonSkilledCarePlans", "Stop active non-skilled care plans?", "enum", "", booleanOptions);

	addProperty("transferDate", "Transfer Date", "date");
	addProperty("transferReason", "Transfer Reason", "tableItem", "Tables.PatientDCReason");
	addProperty("rocDate", "ROC Date", "date");

	addProperty("hospiceBenefitDate", "Hospice Benefit Date", "date");
	addProperty("lastCertStartDate", "Last Cert Start Date - Transfer Agency", "date");
	addProperty("providerId", "Provider ID# - Transfer Agency", "text");
	addProperty("agencySocDate", "Agency SOC Date", "date");
	addProperty("agencyAdmitReason", "Agency Admit Reason", "tableItem", "Tables.HospiceAdmit");
	addProperty("startingCert", "Start with Cert Period #", "text");
	addProperty("admittedFrom", "[A1802] Admitted From", "tableItem", "admittedFromTable");

	addProperty("dischargeDate", "Discharge Date", "date");
	addProperty("addDischargeOrder", "Add discharge physician order", "enum", "", booleanOptions);
	addProperty("dischargeOrderPhysician", "Physician", "tableItem", "patientPhysicians");
	addProperty("dischargeOrderDiscipline", "Discipline", "tableItem", "Tables.Discipline");
	addProperty("dischargeReason", "Discharge Reason", "tableItem", "Tables.PatientDCReason");
	addProperty("facility", "Transfer/Discharge Facility", "tableItem", "Tables.OutsideTables.Facility");
	addProperty("location", "Transfer/Discharge Location", "tableItem", "Tables.PatientStatusLocation");

	addProperty("deathDate", "Date of Death", "date");
	addProperty("deathTime", "Time of Death", "time");
	addProperty("notificationDate", "Discharge Notification Date", "date");
	addProperty("madeAwareDate", "Date Agency Made Aware", "date");

	addProperty("snDischargePlan", "SN Discharge Plan", "text");
	addProperty("ptDischargePlan", "PT Discharge Plan", "text");
	addProperty("otDischargePlan", "OT Discharge Plan", "text");
	addProperty("stDischargePlan", "ST Discharge Plan", "text");
	addProperty("mswDischargePlan", "MSW Discharge Plan", "text");
	addProperty("ntDischargePlan", "NT Discharge Plan", "text");
	addProperty("rtDischargePlan", "RT Discharge Plan", "text");
	addProperty("mchDischargePlan", "MCH Discharge Plan", "text");
	addProperty("additionalDischargePlan", "Additional Discharge Plan", "text");
	addProperty("dischargeNeeds", "Patient's Discharge Needs", "text");
	addProperty("dischargePreferences", "Patient's Discharge Preferences", "text");
	addProperty("dischargeGoals", "Patient's Discharge Goals", "text");
	addProperty("evaluationResultsDiscussed", "Results of Discharge Evaluation Discussed w/ Patient or Patient's Representative?", "enum", "", booleanOptions);
	addProperty("dischargeTransferArrangements", "Arrangements made for Discharge/Transfer", "text");
	addProperty("dischargeInstructions", "Discharge Instructions", "text");
	addProperty("dischargeSummary", "Discharge Summary", "text");
	
	addProperty("company", "Company", "tableItem", "Tables.Company");

	statusModel = new ObjectModel({ propertyDefinitions: statusProperties });
	Content = statusModel.getObject();

	getTemplate = function(actionValue) {
		var containers = {};
		containers.status = {
			attachPoint: "statusContainer",
			sections: [{
				hideDivider: true,
				rows: [{
					fields: [
						{ name: "status", displayName: "Status:", labelSize: "grandest", noField: true }
					]
				}]
			},
			{}]
		};

		containers.referral = {
			sections: [{
				title: "Referral",
				rows: [{
					fields: [
						{ name: "referralStatus", labelSize: "large" },
						{ name: "referralDate" }
					]
				}]
			}]
		};

		containers.soc = {
			sections: [{
				title: "SOC/Admit",
				rows: [{
					fields: [
						{ name: "socDate", labelSize: "large" },
						{ name: "anticipatedFirstVisit", labelSize: "large" }
					]
				}]
			}]
		};

		containers.rns = {
			sections: [{
				title: "RNS",
				rows: [{
					fields: [
						{ name: "rnsDate", labelSize: "large" },
						{ name: "rnsReason" }
					]
				}]
			}]
		};

		containers.sna = {
			sections: [{
				title: "SNA",
				rows: [{
					fields: [
						{ name: "snaDate", labelSize: "large" },
						{ name: "snaReason" }
					]
				}]
			}]
		};

		containers.transfers = {
			sections: [{
				title: "Transfer",
				rows: [{
					fields: [
						{ name: "transferDate", labelSize: "large" },
						{ name: "rocDate", labelSize: "large" }
					]}
				]
			}]
		};

		containers.facilityLocation = {
			attachPoint: "facilityLocationContainer",
			sections: [{
				hideDivider: true,
				rows: [
					{ field: { name: "facility", labelSize: "grand" } },
					{ field: { name: "location", labelSize: "grand" } }
				]
			}]
		};

		containers.discharge = {
			sections: [{
				title: "Discharge",
				rows: [
					{
						orientation: "vertical",
						fields: [
							{ name: "dischargeDate", labelSize: "largest", fieldSize: "large" },
							{ name: "dischargeReason", labelSize: "largest" }
						]
					},
					{
						fields: [
							{ name: "deathDate", labelSize: "largest", fieldSize: "large" },
							{ name: "deathTime", labelSize: "largest" }
						]
					},
					{
						fields: [
							{ name: "facility", labelSize: "largest", fieldSize: "large" },
							{ name: "location", labelSize: "largest" }
						]
					},
					{
						fields: [
							{ name: "notificationDate", labelSize: "largest", fieldSize: "large" },
							{ name: "madeAwareDate", labelSize: "largest" }
						]
					}
				]
			}]
		};
		containers.dischargePlans = {
			attachPoint: "dischargePlansContainer",
			sections: [{
				title: "Discharge Plans",
				hideDivider: true,
				rows: [
					{ field: { name: "snDischargePlan",					labelSize: "grandest", fieldSize: "grandest" } },
					{ field: { name: "ptDischargePlan",					labelSize: "grandest", fieldSize: "grandest" } },
					{ field: { name: "otDischargePlan",					labelSize: "grandest", fieldSize: "grandest" } },
					{ field: { name: "stDischargePlan",					labelSize: "grandest", fieldSize: "grandest" } },
					{ field: { name: "mswDischargePlan",				labelSize: "grandest", fieldSize: "grandest" } },
					{ field: { name: "ntDischargePlan",					labelSize: "grandest", fieldSize: "grandest" } },
					{ field: { name: "rtDischargePlan",					labelSize: "grandest", fieldSize: "grandest" } },
					{ field: { name: "mchDischargePlan",				labelSize: "grandest", fieldSize: "grandest" } },
					{ field: { name: "additionalDischargePlan",			labelSize: "grandest", fieldSize: "grandest" } },
					{ field: { name: "dischargeNeeds",					labelSize: "grandest", fieldSize: "grandest" } },
					{ field: { name: "dischargePreferences",			labelSize: "grandest", fieldSize: "grandest" } },
					{ field: { name: "dischargeGoals",					labelSize: "grandest", fieldSize: "grandest" } },
					{ field: { name: "evaluationResultsDiscussed",		labelSize: "grander", fieldSize: "grandest" } },
					{ field: { name: "dischargeTransferArrangements",	labelSize: "grandest", fieldSize: "grandest" } },
					{ field: { name: "dischargeInstructions",			labelSize: "grandest", fieldSize: "grandest" } },
					{ field: { name: "dischargeSummary",				labelSize: "grandest", fieldSize: "grandest" } }
				]
			}]
		};

		containers.hospice = {
			attachPoint: "hospiceContainer",
			sections: [{
				title: "Hospice",
				rows: [
					{ field: { name: "hospiceBenefitDate", labelSize: "grand" } },
					{
						fields: [
							{ name: "lastCertStartDate", labelSize: "grand" },
							{ name: "providerId", labelSize: "largest" }
						]
					},
					{
						fields: [
							{ name: "agencySocDate", labelSize: "grand" },
							{ name: "agencyAdmitReason", labelSize: "largest" }
						]
					},
					{
						fields: [
							{ name: "startingCert", labelSize: "grand" },
							{ name: "admittedFrom", labelSize: "largest" }
						]
					}
				]
			}]
		};

		containers.locationOfCare = {
			attachPoint: "locationOfCareContainer",
			sections: [{
				title: "Location Of Care",
				rows: [
					{ field: { name: "company", labelSize: "large" } }
				]
			}]
		};

		var template = {};
		if (actionValue === "View") {
			template.containers = [];
			//template.containers.push(containers.referral); // Removed for now
			template.containers.push(containers.soc);
			if (patientInfo.patientStatus === 4 && patientInfo.rnsReason !== "") {
				template.containers.push(containers.rns);
			}
			if (patientInfo.patientStatus === 5) {
				template.containers.push(containers.sna);
			}
			template.containers.push(containers.transfers);
			template.containers.push(containers.discharge);
			containers.dischargePlans.sections[0].title = "Discharge Plans";
			template.containers.push(containers.dischargePlans);
			if (patientType === "Hospice") {
				template.containers.push(containers.hospice);
			}
			if (useLocationOfCare) {
				template.containers.push(containers.locationOfCare);
			}
		} else if (actionValue === "Modify") {
			var patientStatus = patientInfo.patientStatus;
			var transferDate = locale.parse(patientInfo.transferDate, { selector: "date", datePattern: "yyyy-MM-dd" });
			var rocDate = locale.parse(patientInfo.rocDate, { selector: "date", datePattern: "yyyy-MM-dd" });

			template.containers = [];
			template.containers.push({
				sections: [{
					hideDivider: true,
					rows: [
						{ field: { name: "anticipatedFirstVisit", labelSize: "grand" } }
					]
				}]
			});

			// template.containers.push({
			// 	sections: [{
			// 		hideDivider: true,
			// 		rows: [
			// 			{ field: { name: "facility", labelSize: "grand" } },
			// 			{ field: { name: "location", labelSize: "grand" } }
			// 		]
			// 	}]
			// });

			// transfer
			template.containers.push({
				sections: [{
					hideDivider: true,
					rows: [
						{ field: { name: "transferDate", labelSize: "grand" } },
						{ field: { name: "transferReason", labelSize: "grand" } },
						{ field: { name: "rocDate", labelSize: "grand" } }
					]
				}]
			});

			if (patientStatus != 7 && transferDate > rocDate) {
				template.containers.push(containers.facilityLocation);
				template.containers.push(containers.dischargePlans);
			}

			if (patientStatus == 4 && patientInfo.rnsReason) {
				template.containers.push({
					sections: [{
						title: "RNS",
						hideDivider: true,
						rows: [
							{ field: { name: "rnsDate", labelSize: "grand" } },
							{ field: { name: "rnsReason", labelSize: "grand" } },
							{ field: { name: "errorMedications", labelSize: "grand" } },
							{ field: { name: "errorOrders", labelSize: "grand" } },
							{ field: { name: "deleteScheduledMeetings", labelSize: "grand" } },
							{ field: { name: "rnsSnaNarrative", labelSize: "large", fieldSize: "grandest" } }
						]
					}]
				});
				//template.containers.push(containers.facilityLocation);
			} else if (patientStatus == 5) {
				template.containers.push({
					sections: [{
						title: "SNA",
						hideDivider: true,
						rows: [
							{ field: { name: "snaDate", labelSize: "grand" } },
							{ field: { name: "snaReason", labelSize: "grand" } },
							{ field: { name: "errorMedications", labelSize: "grand" } },
							{ field: { name: "errorOrders", labelSize: "grand" } },
							{ field: { name: "deleteScheduledMeetings", labelSize: "grand" } },
							{ field: { name: "rnsSnaNarrative", labelSize: "large", fieldSize: "grandest" } }
						]
					}]
				});
				//template.containers.push(containers.facilityLocation);
			} else if (patientStatus == 7) {
				template.containers.push({
					sections: [{
						title: "Discharge",
						hideDivider: true,
						rows: [
							{
								orientation: "vertical",
								fields: [
									{ name: "dischargeDate", labelSize: "grand", fieldSize: "large" },
									{ name: "dischargeReason", labelSize: "largest" }
								]
							},
							{
								fields: [
									{ name: "deathDate", labelSize: "grand", fieldSize: "large" },
									{ name: "deathTime", labelSize: "largest" }
								]
							},
							{
								fields: [
									{ name: "notificationDate", labelSize: "grand", fieldSize: "large" },
									{ name: "madeAwareDate", labelSize: "largest" }
								]
							},
							{ field: { name: "deleteScheduledMeetings", labelSize: "grand" } }
						]
					}]
				});
				template.containers.push(containers.facilityLocation);
				template.containers.push(containers.dischargePlans);
			}

			if (patientType === "Hospice") {
				template.containers.push(containers.hospice);
			}
			if (useLocationOfCare) {
				template.containers.push(containers.locationOfCare);
			}
		} else if (actionValue === "Admit To Hospice") {
			template.containers = [containers.hospice];
			if (useLocationOfCare) {
				template.containers.push(containers.locationOfCare);
			}
		} else if (actionValue === "RNS") {
			template = {
				containers: [{
					sections: [{
						hideDivider: true,
						rows: [
							{ field: { name: "rnsDate", labelSize: "grand" } },
							{ field: { name: "rnsReason", labelSize: "grand" } },
							//{ field: { name: "facility", labelSize: "grand" } },
							//{ field: { name: "location", labelSize: "grand" } },
							{ field: { name: "deleteScheduledVisits", labelSize: "grand" } },
							{ field: { name: "deleteScheduledMeetings", labelSize: "grand" } },
							{ field: { name: "errorMedications", labelSize: "grand" } },
							{ field: { name: "errorOrders", labelSize: "grand" } },
							{ field: { name: "rnsSnaNarrative", labelSize: "large", fieldSize: "grandest" } }
						]
					}]
				}]
			};
		} else if (actionValue === "SNA") {
			template = {
				containers: [{
					sections: [{
						hideDivider: true,
						rows: [
							{ field: { name: "snaDate", labelSize: "grand" } },
							{ field: { name: "snaReason", labelSize: "grand" } },
							//{ field: { name: "facility", labelSize: "grand" } },
							//{ field: { name: "location", labelSize: "grand" } },
							{ field: { name: "deleteScheduledVisits", labelSize: "grand" } },
							{ field: { name: "deleteScheduledMeetings", labelSize: "grand" } },
							{ field: { name: "errorMedications", labelSize: "grand" } },
							{ field: { name: "errorOrders", labelSize: "grand" } },
							{ field: { name: "rnsSnaNarrative", labelSize: "large", fieldSize: "grandest" } }
						]
					}]
				}]
			};
		} else if (actionValue === "Transfer") {
			template = {
				containers: [{
					sections: [{
						hideDivider: true,
						rows: [
							{ field: { name: "anticipatedFirstVisit", labelSize: "grand" } },
							{ field: { name: "transferDate", labelSize: "grand" } },
							{ field: { name: "transferReason", labelSize: "grand" } },
							{ field: { name: "madeAwareDate", labelSize: "grand" } },
							{ field: { name: "facility", labelSize: "grand" } },
							{ field: { name: "location", labelSize: "grand" } },
							{ field: { name: "deleteScheduledVisits", labelSize: "grand" } },
							{ field: { name: "deleteScheduledMeetings", labelSize: "grand" } },
							{ field: { name: "discontinueMedications", labelSize: "grand" } },
							{ field: { name: "deleteIncompleteMedications", labelSize: "grand" } },
							{ field: { name: "holdActiveMedications", labelSize: "grand" } },
							{ field: { name: "discontinueOrders", labelSize: "grand" } },
							{ field: { name: "deleteIncompleteOrders", labelSize: "grand" } },
							{ field: { name: "holdActiveOrders", labelSize: "grand" } }
						]
					}]
				}]
			};

			containers.dischargePlans.sections[0].title = null;
			template.containers.push(containers.dischargePlans);
		} else if (actionValue === "ROC") {
			template = {
				containers: [{
					sections: [{
						hideDivider: true,
						rows: [
							{ field: { name: "anticipatedFirstVisit", labelSize: "grand" } },
							{ field: { name: "rocDate", labelSize: "grand" } },
							{ field: { name: "resumeMeds", labelSize: "grand" } },
							{ field: { name: "resumeOrders", labelSize: "grand" } }
						]
					}]
				}]
			};
			if (useLocationOfCare) {
				template.containers.push(containers.locationOfCare);
			}
		} else if (actionValue === "Cancel Patient") {
			template = {
				containers: [{
					sections: [{
						hideDivider: true,
						rows: [
							{
								field: {
									name: "confirmCancelPatientMatch",
									displayName: "Please confirm the following details match the patient you wish to cancel:",
									labelSize: "grandest",
									noField: true
								}
							},
							{ field: { name: "referralDate", labelSize: "large" } },
							{ field: { name: "socDate", labelSize: "large" } },
							{ field: { name: "dischargeDate", labelSize: "large" } },
							{
								field: {
									name: "cancelPatientWarning",
									displayName: "<br><br><strong>This action cannot be undone.</strong><br>" +
												 "All charted visits and scheduled visits for this patient will be permanently deleted.<br>" + 
												 "All unsynchronized charting on clients will be lost.<br>" +
												 "This account will only be available to select during patient registration and have data carried forward to the new account registered.",
									labelSize: "grandest",
									noField: true
								}
							}
						]
					}]
				}]
			};
		} else if (actionValue === "Discharge") {
			template = {
				containers: [{
					sections: [
						{
							hideDivider: true,
							rows: [
								{
									fields: [
										{ name: "dischargeDate", labelSize: "grand", fieldSize: "large" }
									]
								},
								{ field: { name: "addDischargeOrder", labelSize: "grand" } },
								{
									fields: [
										{ name: "dischargeOrderPhysician", labelSize: "grand" },
										{ name: "dischargeOrderDiscipline" }
									]
								},
								{ field: { name: "dischargeReason", labelSize: "grand" } },
								{
									fields: [
										{ name: "deathDate", labelSize: "grand" },
										{ name: "deathTime", labelSize: "largest" }
									]
								},
								{
									fields: [
										{ name: "facility", labelSize: "grand" },
										{ name: "location", labelSize: "largest" }
									]
								},
								{
									fields: [
										{ name: "notificationDate", labelSize: "grand" },
										{ name: "madeAwareDate", labelSize: "largest" }
									]
								}
							]
						},
						{
							hideDivider: true,
							rows: [
								{ field: { name: "deleteScheduledVisits", labelSize: "grand" } },
								{ field: { name: "deleteScheduledMeetings", labelSize: "grand" } },
								{ field: { name: "discontinueMedications", labelSize: "grand" } },
								{ field: { name: "deleteIncompleteMedications", labelSize: "grand" } },
								{ field: { name: "discontinueOrders", labelSize: "grand" } },
								{ field: { name: "deleteIncompleteOrders", labelSize: "grand" } },
								{ field: { name: "stopNonSkilledCarePlans", labelSize: "grand" } }
							]
						},
						{
							hideDivider: true,
							rows: [
								{ field: { name: "snDischargePlan", labelSize: "grandest", fieldSize: "grandest" } },
								{ field: { name: "ptDischargePlan", labelSize: "grandest", fieldSize: "grandest" } },
								{ field: { name: "otDischargePlan", labelSize: "grandest", fieldSize: "grandest" } },
								{ field: { name: "stDischargePlan", labelSize: "grandest", fieldSize: "grandest" } },
								{ field: { name: "mswDischargePlan", labelSize: "grandest", fieldSize: "grandest" } },
								{ field: { name: "ntDischargePlan", labelSize: "grandest", fieldSize: "grandest" } },
								{ field: { name: "rtDischargePlan", labelSize: "grandest", fieldSize: "grandest" } },
								{ field: { name: "mchDischargePlan", labelSize: "grandest", fieldSize: "grandest" } },
								{ field: { name: "additionalDischargePlan", labelSize: "grandest", fieldSize: "grandest" } },
								{ field: { name: "dischargeNeeds", labelSize: "grandest", fieldSize: "grandest" } },
								{ field: { name: "dischargePreferences", labelSize: "grandest", fieldSize: "grandest" } },
								{ field: { name: "dischargeGoals", labelSize: "grandest", fieldSize: "grandest" } },
								{ field: { name: "evaluationResultsDiscussed", labelSize: "grander" } },
								{ field: { name: "dischargeTransferArrangements", labelSize: "grandest", fieldSize: "grandest" } },
								{ field: { name: "dischargeInstructions", labelSize: "grandest", fieldSize: "grandest" } }
							]
						},
						{
							hideDivider: true,
							rows: [
								{ field: { name: "dischargeSummary", labelSize: "grandest", fieldSize: "grandest" } }
							]
						}
					]
				}]
			};
		} else if (actionValue === "Unfile Referral") {
			template = {
				containers: [{
					sections: [{
						hideDivider: true,
						rows: [{
							field: {
								name: "unfileReferralMessage",
								displayName: "Please confirm the referral should be unfiled.",
								labelSize: "grandest",
								noField: true
							}
						}]
					}]
				}]
			};
		} else if (actionValue === "Cancel Admit") {
			template = {
				containers: [{
					sections: [{
						hideDivider: true,
						rows: [
							{ field: { name: "rnsDate", labelSize: "large" } },
							{ field: { name: "rnsReason", labelSize: "large" } },
							{ field: { name: "snaDate", labelSize: "large" } },
							{ field: { name: "snaReason", labelSize: "large" } }
						]
					}]
				}]
			};
		} 
		
		// DEPRECATED ND-9045
		/*else if (actionValue === "Cancel RNS") {
			template = {
				containers: [{
					sections: [{
						hideDivider: true,
						rows: [{
							field: {
								name: "cancelRnsMessage",
								displayName: "To cancel RNS status, click Save.",
								labelSize: "grandest",
								noField: true
							}
						}]
					}]
				}]
			};
		} else if (actionValue === "Cancel SNA") {
			template = {
				containers: [{
					sections: [{
						hideDivider: true,
						rows: [{
							field: {
								name: "cancelSnaMessage",
								displayName: "To cancel SNA status, click Save.",
								labelSize: "grandest",
								noField: true
							}
						}]
					}]
				}]
			};
		}*/ 
		
		else if (actionValue === "Cancel Discharge") {
			template = {
				containers: [{
					sections: [{
						hideDivider: true,
						rows: [{
							field: {
								name: "cancelDischargeMessage",
								displayName: "<p>This patient may have discontinued orders including medications that need review or reactivation.</p>" + 
											 "<p>To cancel this patient's discharge, click Save.</p>",
								labelSize: "grandest",
								noField: true
							}
						}]
					}]
				}]
			};
		}

		if (actionValue !== "View") {
			// Insert the status container at the top of all non-view containers
			//template.containers.splice(0, 0, containers.status);
		}

		return template;
	};

	var StatusEditor = declare([ObjectEditor], {
		objectModel: statusModel,
		templateString: null,
		invalidDateText: "The value entered is not valid.",
		useLocationOfCare: false,

		postMixInProperties: function() {
			var t = this;
			t.inherited("postMixInProperties", arguments);

			// Turn narrative fields into text areas
			arrayUtils.forEach(narrativeFields, function(narrativeField) {
				var propertyModel = t.getPropertyModel(narrativeField);
				propertyModel.getField = function(node) { return new Textarea(this.getFieldProperties(), node); };
				propertyModel.maxLength = 4500;
			});

			// Prevent yes/no radio buttons from defaulting
			t.objectModel._propertyModelStore.query({ type: "enum" }).forEach(function(propertyModel) {
				propertyModel.noDefault = true;
			});

			if (t.actionType === "View") {
				arrayUtils.forEach(t.objectModel.getPropertyModels(), function(propertyModel) {
					propertyModel.fieldProperties.disabled = true;
				});
			}
		},

		buildRendering: function() {
			var t = this;
			t.inherited("buildRendering", arguments);
			
			if (!t.locWidget && t.hospiceContainer) {
				t.locWidget = new LevelOfCareWidget({statusEditorId: this.id});
				t.hospiceContainer.appendChild(t.locWidget.domNode);
			}
			if (t.useLocationOfCare) {
				if (!t.locationOfCareWidget && t.locationOfCareContainer) {
					t.locationOfCareWidget = new LocationOfCareWidget({statusEditorId: this.id});
					t.locationOfCareContainer.appendChild(t.locationOfCareWidget.domNode);
				}
			}

			if (t.startingCertField) {
				t.startingCertField.set("regExp", numberRegex);
				domStyle.set(t.startingCertField.domNode, "width", "45px");
			}

			// Set up discharge plans expand area in view
			if (t.actionType === "View" && t.dischargePlansContainer) {
				var nodes = query(".content-area-title-text", t.dischargePlansContainer);
				var titleContainer = arrayUtils.filter(nodes, function(node) {
					return node.innerHTML === "Discharge Plans";
				})[0];
				titleContainer.innerHTML = "";
				var link = domConstruct.toDom("<a>+ Discharge Plans</a>");
				domConstruct.place(link, titleContainer, "empty");

				var expandableFields = ["snDischargePlan", "ptDischargePlan", "otDischargePlan", "stDischargePlan", "mswDischargePlan", "ntDischargePlan",
										"rtDischargePlan", "mchDischargePlan", "additionalDischargePlan", "dischargeNeeds", "dischargePreferences",
										"dischargeGoals", "evaluationResultsDiscussed", "dischargeTransferArrangements", "dischargeInstructions"];

				t.displayDischargePlans = function(display) {
					link.innerHTML = (display ? "-" : "+")+" Discharge Plans";
					arrayUtils.forEach(expandableFields, function(field) {
						domStyle.set(t[field+"Container"], "display", (display ? "" : "none"));
					});
				};
				t.displayDischargePlans(false);
				t.own(on(link, "click", function() {
					t.displayDischargePlans(link.innerHTML.indexOf("+") != -1);
				}));
			}
		},

		postCreate: function() {
			var t = this;
			t.inherited(arguments);

			// If a modification to one field means another field changes from invalid to valid, update object with valid value
			arrayUtils.forEach(t.objectModel.getPropertyModels(), function(propertyModel) {
				var widget = t[propertyModel.name + "Field"];
				if (widget && (typeof widget.state !== "undefined")) {
					widget.watch("state", function(name, oldValue, value) {
						// state: ""=Normal, Incomplete, or Error
						// Don't fire for field in focus (this will need to be changed for async use)
						if (!t.inObjectLoad && !widget.get("focused") && !widget.get("disabled") && (oldValue != "") && (value == "") && (t.object.get(propertyModel.name) != widget.get("value"))) {
							t.object._changeAttrValue(propertyModel.name, widget.get("value"));
							t._postModify();
						}
					});
				}
			});

			t.own(aspect.before(t, "_setObjectAttr", function(object) {
				if (t.actionValue === "ROC") {
					object.rocDate = "";
					if (!settings.roc.carryForwardAnticipated1stVisitDate) {
						object.anticipatedFirstVisit = "";
					}
				} else if (t.actionValue === "Transfer") {
					object.transferDate = "";
					object.transferReason = "";
					object.anticipatedFirstVisit = "";
					object.facility = "";
					object.location = "";
				} else if (t.actionValue === "Discharge") {
					object.facility = "";
					object.location = "";
				}
			}));

			t.own(aspect.after(t, "_setObjectAttr", function(object) {
				// Narrative labels go above fields
				arrayUtils.forEach(narrativeFields, function(narrativeField) {
					if (t[narrativeField + "Container"]) {
						domClass.add(t[narrativeField + "Container"], "vertical-field-align");
					}
				});

				// Add statement button to discharge plan fields
				if (t.actionType !== "View") {
					//console.time("Add Statement buttons");
					arrayUtils.forEach(dischargePlanFields, function(dischargePlanField) {
						if (t[dischargePlanField + "Field"]) {
							var statementNode = domConstruct.create("input", {}, t[dischargePlanField + "LabelContainer"]);

							var statementMenu = new DropDownMenu();
							t.statementButton = new DropDownButton({
								label: "Add Statement",
								dropDown: statementMenu,
								style: "float:right",
								onClick: function() {
									this.focus();
								}
							}, statementNode);

							if (discPlanStatementsTableData.children.length > 0) {
								var maxLength = t.getPropertyModel(dischargePlanField).maxLength;

								var addTextToOrder = function(text) {
									var currentText = t.object.get(dischargePlanField);
									text = (currentText === "" ? text : " " + text);
									var textLength = (currentText + text).length;
									if (textLength > maxLength) {
										controller.showMessage("The discharge plan text is too long to append this statement. Please shorten the text by " + (textLength - maxLength) + " characters.");
									} else {
										t.object.set(dischargePlanField, currentText + text);
										t._postModify();
									}
								};

								arrayUtils.forEach(discPlanStatementsTableData.children, function(statement) {
									var newOption = new MenuItem({
										label: escapeText(statement.name.toString()),
										onClick: function() {
											addTextToOrder(statement.text);
										}
									});
									statementMenu.addChild(newOption);
								});
							} else {
								statementMenu.addChild(new MenuItem({ label: "No statements found!" }));
							}
						}
					});
					//console.timeEnd("Add Statement buttons");
				}

				t.actionValueLogic();

				// Don't show ROC date in view mode if there is a later transfer date
				if (t.actionType == "View") {
					var rocDate = t.object.get("rocDate");
					var transferDate = t.object.get("transferDate");
					if (transferDate && rocDate && rocDate < transferDate) {
						t.object.set("rocDate", "");
					}
				}

				if (t.notificationDateField) {
					// The Discharge Notification Date cannot be a future date (for non-training patients) or before the SOC date
					if (!patientInfo.trainingPatient) {
						t.notificationDateField.constraints.max = new Date();
					}
					if (t.object.get("socDate")) {
						t.notificationDateField.constraints.min = t.object.get("socDate");
					}
				}

				if (t.madeAwareDateField) {
					// The Date Agency Made Aware cannot be a future date (for non-training patients) or before the SOC date
					if (!patientInfo.trainingPatient) {
						t.madeAwareDateField.constraints.max = new Date();
					}
					if (t.object.get("socDate")) {
						t.madeAwareDateField.constraints.min =  t.object.get("socDate");
					}
				}

				if (t.deathDateField) {
					// Date of Death cannot be a future date (for non-training patients) or before the SOC date
					if (!patientInfo.trainingPatient) {
						t.deathDateField.constraints.max = new Date();
					}
					if (t.object.get("socDate")) {
						t.deathDateField.constraints.min = t.object.get("socDate");
					}
				}

				if (t.rocDateField) {
					// ROC Date cannot be a future date (for non-training patients).
					if (!patientInfo.trainingPatient) {
						t.rocDateField.constraints.max = new Date();
					}
				}

				if (t.rnsDateField) {
					// RNS date cannot be more than 5 days in the future.
					t.rnsDateField.constraints.max = date.add(new Date(), "day", 5);
				}

				var restrictIdArray;
				if (t.rnsReasonField || t.snaReasonField) {
					restrictIdArray = [];
					TableManager.get("Tables.RNSSNAReason").query({ hospice: (patientType === "Hospice") }).forEach(function(reason) {
						restrictIdArray.push(reason.id);
					});
					if (t.rnsReasonField) {
						t.populateSelect(t.rnsReasonField, TableManager.get("Tables.RNSSNAReason"), t.object.get("rnsReason"), true, restrictIdArray);
					}
					if (t.snaReasonField) {
						t.populateSelect(t.snaReasonField, TableManager.get("Tables.RNSSNAReason"), t.object.get("snaReason"), true, restrictIdArray);
					}
				}

				if (t.dischargeReasonField || t.transferReasonField) {
					restrictIdArray = [];
					TableManager.get("Tables.PatientDCReason").query({ hospice: (patientType === "Hospice") }).forEach(function(reason) {
						if (patientType === "Homecare" || reason.hisReasonForDischarge) {
							restrictIdArray.push(reason.id);
						}
					});
					if (t.dischargeReasonField) {
						t.populateSelect(t.dischargeReasonField, TableManager.get("Tables.PatientDCReason"), t.object.get("dischargeReason"), true, restrictIdArray);
					}
					if (t.transferReasonField) {
						t.populateSelect(t.transferReasonField, TableManager.get("Tables.PatientDCReason"), t.object.get("transferReason"), true, restrictIdArray);
					}
				}

				if (t.companyField) {
					restrictIdArray = [];
					TableManager.get("Tables.Company").query((patientType === "Homecare" ? {companyType: 1} : (patientType === "Hospice" ? {companyType: 2} : null))).forEach(function(company) {
						restrictIdArray.push(company.id);
					});
					t.populateSelect(t.companyField, TableManager.get("Tables.Company"), t.object.get("company"), true, restrictIdArray);
				}

				if (object.get("agencySocDate")) {
					// Hospice admit date can't be removed
					t.require("agencySocDate");
				}
			}, true));

			t.registerChangeLogic("addDischargeOrder", function(value) {
				value = +value;
				t.disable("dischargeOrderPhysician", !value);
				t.disable("dischargeOrderDiscipline", !value);
				t.require("dischargeOrderPhysician", value);
				t.require("dischargeOrderDiscipline", value);
				t.object.set("dischargeOrderPhysician", "");
				t.object.set("dischargeOrderDiscipline", "");
			});

			t.registerChangeLogic("dischargeReason", function(value) {
				var reasonObj = TableManager.get("Tables.PatientDCReason").get(value);
				var deceased = null;
				var planned = null;
				if (typeof reasonObj !== "undefined") {
					deceased = reasonObj.deceased;
					planned = reasonObj.planned;
				}
				var hospicePatient = (patientType === "Hospice");

				t.require("madeAwareDate", (!!value && (planned === 0)));

				if (deceased) {
					t.disable("deathDate", false);
					t.disable("deathTime", false);
					t.require("deathDate", true);
					if (hospicePatient) {
						t.require("deathTime", true);
					}
				} else {
					t.require("deathDate", false);
					t.require("deathTime", false);
					if (!hospicePatient) {
						t.disable("deathDate", true);
						t.disable("deathTime", true);
					}
					if (!t.inObjectLoad) {
						t.object.set("deathDate", "");
						t.object.set("deathTime", "");
					}
				}
			});

			t.registerChangeLogic("transferReason", function(value) {
				// ValTRFREA
				var reasonObj = TableManager.get("Tables.PatientDCReason").get(value);
				var planned = null;

				if (typeof reasonObj !== "undefined") {
					planned = reasonObj.planned;
				}

				t.require("madeAwareDate", (!!value && planned === 0));
			});

			t.registerChangeLogic("deathDate", function(value) {
				// if today and not training patient, set death time constraint max to now
				if (t.deathTimeField) {
					var today = new Date();
					today.setHours(0, 0, 0, 0);
					var constraints = t.deathTimeField.get("constraints");
					if ((!patientInfo.trainingPatient) && value && (value.getTime() == today.getTime())) {
						constraints.max = new Date();
					} else {
						delete constraints.max;
					}

					t.deathTimeField.set("constraints", constraints);
				}
			});

			t.registerChangeLogic("facility", function(value) {
				var facilityObj = TableManager.get("Tables.OutsideTables.Facility").get(value);
				var location = "";
				if (facilityObj && facilityObj.PatientStatusLocation) {
					location = facilityObj.PatientStatusLocation;
					t.disable("location", true);
				} else {
					t.disable("location", false);
				}

				if (!t.inObjectLoad) {
					t.object.set("location", location);
				}
			});

			t.registerChangeLogic("dischargeDate", function(value) {
				// ValDCPDT
				var today = new Date();
				today.setHours(0, 0, 0, 0);
				if (!t.inObjectLoad) {
					if (date.difference(value, today, "day") > 335) {
						actionShowMessage("WARNING. The patient discharge date is more than 11 months in the past. Please verify before proceeding.");	
					}

					if (settings.employeeActivitySite && t.dischargeDateField && t.deleteScheduledVisitsField && t.deleteScheduledMeetingsField) {
						serverRequest("Patient.Status.Utility", "FutureActivitiesCheck", {ndocNumber: ndocNumber, date: value}).then(function(results) {
							t.require("deleteScheduledVisits", results.unchartedScheduledVisits);
							t.disable("deleteScheduledVisits", !results.unchartedScheduledVisits);
	
							t.require("deleteScheduledMeetings", results.scheduledMeetings);
							t.disable("deleteScheduledMeetings", !results.scheduledMeetings);
						});
					}

					serverRequest("Patient.Status.Utility", "ActiveNonSkilledCarePlans", {ndocNumber: ndocNumber, date: value}).then(function(results) {
						t.disable("stopNonSkilledCarePlans", !results);
						if (!results) {
							t.object.set("stopNonSkilledCarePlans", "0");
						}
					});

					
				}
			});

			t.registerChangeLogic("transferDate", function(value) {
				// ValTRFDT

				if (!t.inObjectLoad) {
					var admitDate = locale.parse(patientInfo.admitDate, { selector: "date", datePattern: "yyyy-MM-dd" });
					var socDate = t.object.get("socDate");
					var anticipatedFirstVisit = t.getValidationValue("anticipatedFirstVisit");

					if (value) {
						if (anticipatedFirstVisit && anticipatedFirstVisit < value && t.actionValue === "Transfer" && patientInfo.switchoverPatient) {
							actionShowMessage("WARNING: The anticipated 1st visit date is prior to the Transfer date.");
						} else if (anticipatedFirstVisit && anticipatedFirstVisit < value && t.actionValue !== "Transfer" && t.actionValue !== "Modify") {
							actionShowMessage("WARNING: The anticipated 1st visit date is prior to the Transfer date.");
						} else if (value < socDate && patientInfo.switchoverPatient) {
							actionShowMessage("WARNING: Transfer Date is before the SOC date.");
						} else if (value < admitDate && patientInfo.switchoverPatient) {
							actionShowMessage("WARNING: Transfer Date is before the admit date.");
						}
					}
				}

				if (t.actionValue !== "Modify") {
					t.require("transferReason", value);
					t.disable("transferReason", !value);
				}

				if (settings.employeeActivitySite && t.transferDateField && t.deleteScheduledVisitsField && t.deleteScheduledMeetingsField) {
					t.disable("deleteScheduledVisits", true);
					t.disable("deleteScheduledMeetings", true);
					
					serverRequest("Patient.Status.Utility", "FutureActivitiesCheck", {ndocNumber: ndocNumber, date: value}).then(function(results) {
						if (settings.transfer.allowDeleteScheduledVisits) {
							t.require("deleteScheduledVisits", results.unchartedScheduledVisits);
							t.disable("deleteScheduledVisits", !results.unchartedScheduledVisits);
						} else {
							domStyle.set(t.deleteScheduledVisitsContainer, "display", "none");
						}

						t.require("deleteScheduledMeetings", results.scheduledMeetings);
						t.disable("deleteScheduledMeetings", !results.scheduledMeetings);
					});
				}
			});

			t.registerChangeLogic("rocDate", function(value) {
				// ValROCDT
				var anticipatedFirstVisit = t.getValidationValue("anticipatedFirstVisit");

				if (!t.inObjectLoad && value) {
					if (anticipatedFirstVisit && anticipatedFirstVisit < value && t.actionValue !== "ROC") {
						actionShowMessage("WARNING: The anticipated 1st visit date is prior to the ROC date.");
					}
					// Hard stop if action == ROC in rocDateField.validator()
				}
			});

			t.registerChangeLogic("hospiceBenefitDate", function(value) {
				// ValBENEFITDT

				if (!t.inObjectLoad) {
					var socDate = t.getValidationValue("socDate");
					var agencySocDate = t.getValidationValue("agencySocDate");
					var lastCertStartDate = t.getValidationValue("lastCertStartDate");
					var startingCert = t.getValidationValue("startingCert");

					if (!agencySocDate) {
						agencySocDate = socDate;  // TODO: Verify this - should be after first check below?
					}

					if (agencySocDate && value > agencySocDate) {
						actionShowMessage("Warning: The Hospice Benefit Date is after the Agency SOC Date.");
					} else if (lastCertStartDate && value > lastCertStartDate) {
						actionShowMessage("Warning: The Hospice Benefit Date is after the Last Cert Start Date-Transfer Agency.");
					} else if (startingCert) {
						var days;
						if (lastCertStartDate) {
							days = date.difference(lastCertStartDate, value, "day");
						} else if (agencySocDate) {
							days = date.difference(agencySocDate, value, "day");
						}

						if (days) {
							if ((days > 180 && startingCert < 3) || (days < 180 && startingCert > 2)) {
								if (lastCertStartDate) {
									actionShowMessage("Warning: 'Start with Cert Period #' is " + startingCert + " and the difference between the 'Hospice Benefit Date' and the 'Last Cert Start Date-Transfer Agency' is " + days + " days.");
								} else if (agencySocDate) {
									actionShowMessage("Warning: 'Start with Cert Period #' is " + startingCert + " and the difference between the 'Hospice Benefit Date' and the 'Agency SOC Date' is " + days + " days.");
								}
							}
						}
					}
				}
			});

			t.registerChangeLogic("agencySocDate", function(value) {
				// ValHOSPDT
				t.disable("agencyAdmitReason", !value);
				t.disable("startingCert", !value);
				t.require("agencyAdmitReason", value);
				t.require("startingCert", value);

				if (value) {
					t.object.set("socDate", value);
				}
			});

			t.registerChangeLogic("startingCert", function(value) {
				// ValCERTNUM
				if (t.agencySocDateField) {
					t.agencySocDateField.validate();
				}
			});

			t.registerChangeLogic("discontinueMedications" , function(value) {
				if (t.actionValue === "Transfer" && value == 1) {
					t.object.set("holdActiveMedications", "0");
				}
			});

			t.registerChangeLogic("holdActiveMedications" , function(value) {
				if (t.actionValue === "Transfer" && value == 1) {
					t.object.set("discontinueMedications", "0");
				}
			});

			t.registerChangeLogic("discontinueOrders" , function(value) {
				if (!t.inObjectLoad && value == 1) {
					if (t.actionValue === "Transfer") {
						t.object.set("holdActiveOrders", "0");
					}
					actionShowMessage("If you require a Physician Order for MD confirmation, please chart the order AFTER you completed/SAVED this " + (t.actionValue === "Transfer" ? "Transfer" : "Patient Discharge") + ".");
				}
			});

			t.registerChangeLogic("holdActiveOrders" , function(value) {
				if (!t.inObjectLoad && value == 1) {
					if (t.actionValue === "Transfer") {
						t.object.set("discontinueOrders", "0");
					}
					actionShowMessage("If you require a Physician Order for MD confirmation, please chart the order AFTER you completed/SAVED this " + (t.actionValue === "Transfer" ? "Transfer" : "Patient Discharge") + ".");
				}
			});

			t.registerChangeLogic("anticipatedFirstVisit", function(value) {
				if (!t.inObjectLoad) {
					var validation = anticipatedFirstVisitValidation(value);
					if (validation.message) {
						actionShowMessage(validation.message);
					}
				}
			});

			if (t.agencySocDateField) {
				t.agencySocDateField.validator = function() {
					// ValHOSPDT
					var value = this.get("value");
					var valid = this.inherited("validator", arguments);
					var originalValue = t.objectCopy.get("agencySocDate");
					var valueChanged = (value && originalValue && value.getTime() != originalValue.getTime());

					if (value && (!originalValue || valueChanged) && valid) {
						var today = new Date();
						today.setHours(0, 0, 0, 0);
						var dischargeDate = t.getValidationValue("dischargeDate");
						var startingCert = t.getValidationValue("startingCert");
						var lastCertStartDate = t.getValidationValue("lastCertStartDate") || locale.parse(patientInfo.lastCertStartDate, { selector: "date", datePattern: "yyyy-MM-dd" });

						var message;
						if (patientInfo.patientStatus === 7 && value > dischargeDate) {
							this.set("invalidMessage", "The Agency SOC Date cannot be after the discharge date.");
							valid = false;
						} else if (!patientInfo.trainingPatient && value > date.add(today, "day", 5)) {
							this.set("invalidMessage", "The Agency SOC Date cannot be more than 5 days in the future.");
							valid = false;
						} else if (lastCertStartDate && lastCertStartDate >= value) {
							this.set("invalidMessage", "The Agency SOC date must be after the Last Cert Start Date-Transfer Agency date.");
							valid = false;
						} else if (!t.object.startingCert && lastCertStartDate && date.difference(lastCertStartDate, value, "day") > 89) {
							message = "If the 'Last Cert Start Date - Transfer Agency' is " + locale.format(lastCertStartDate, { selector: 'date' }) + ", " +
								"then the 'Agency SOC Date' cannot be past " + locale.format(date.add(lastCertStartDate, "day", 89), { selector: 'date' }) + ".";
							this.set("invalidMessage", message);
							valid = false;
						} else if (startingCert && lastCertStartDate) {
							var end;
							var certText;
							if (startingCert == 1) {
								end = date.add(lastCertStartDate, "day", 89);
								certText = "1 (1st 90 Days)";
							} else if (startingCert == 2) {
								end = date.add(lastCertStartDate, "day", 89);
								certText = "2 (2nd 90 Days)";
							} else if (startingCert > 2) {
								end = date.add(lastCertStartDate, "day", 59);
								certText = "greater than 2 (a 60 Day period)";
							}

							if ((value <= lastCertStartDate) || (value > end)) {
								message = "If 'Start with Cert Period #' is " + certText + ", and the 'Last Cert Start Date - Transfer Agency' is " +
									locale.format(lastCertStartDate, { selector: 'date' }) + ", then the only valid 'Agency SOC Date' is " +
									locale.format(date.add(lastCertStartDate, "day", 1), { selector: 'date' }) + " through " + locale.format(end, { selector: 'date' }) + ".";
								this.set("invalidMessage", message);
								valid = false;
							}
						}
					} else {
						this.set("invalidMessage", t.invalidDateText);
					}

					return valid;
				};
			}

			if (t.lastCertStartDateField) {
				t.lastCertStartDateField.validator = function() {
					var value = this.get("value");
					var valid = this.inherited("validator", arguments);
					var originalValue = t.objectCopy.get("lastCertStartDate");
					var valueChanged = (value && originalValue && value.getTime() != originalValue.getTime());

					if (value && (!originalValue || valueChanged) && valid) {
						var agencySocDate = t.getValidationValue("agencySocDate");
						if (!agencySocDate) {
							agencySocDate = locale.parse(patientInfo.socDate, { selector: "date", datePattern: "yyyy-MM-dd" });
						}

						if (agencySocDate && value > agencySocDate) {
							this.set("invalidMessage", "The Last Cert Start Date cannot be after the Agency SOC Date.");
							valid = false;
						} else if (agencySocDate && value.getTime() == agencySocDate.getTime()) {
							this.set("invalidMessage", "The Last Cert Start Date cannot be the same as the Agency SOC Date.");
							valid = false;
						} else if (false) {
							// ValHOSPTRFDT, CERTCHK3
						}
					} else {
						this.set("invalidMessage", t.invalidDateText);
					}
					return valid;
				};
			}

			if (t.startingCertField) {
				t.startingCertField.validator = function() {
					var value = this.get("value");
					var valid = this.inherited("validator", arguments);
					var originalValue = t.objectCopy.get("startingCert");
					var valueChanged = (value && originalValue && value != originalValue);

					if ((value > 2) && (!originalValue || valueChanged) && valid) {
						var lastCertStartDate = t.getValidationValue("lastCertStartDate");
						var socDate = t.object.get("socDate");
						var agencySocDate = t.getValidationValue("agencySocDate");

						if (!agencySocDate) {
							agencySocDate = socDate;
						}

						if (lastCertStartDate && agencySocDate) {
							var days = date.difference(lastCertStartDate, agencySocDate, "day");

							if (days > 59) {
								var message = "If the 'Last Cert Start Date - Transfer Agency' is " + locale.format(lastCertStartDate, { selector: 'date' }) +
									" and the 'Agency SOC Date' is " + locale.format(agencySocDate, { selector: 'date' }) +
									" then the 'Start with Cert Period #' cannot be a 60-day cert period.";
								this.set("invalidMessage", message);
								valid = false;
							}
						}
					} else {
						this.set("invalidMessage", t.invalidDateText);
					}
					return valid;
				};
			}

			if (t.dischargeDateField) {
				t.dischargeDateField.validator = function() {
					var value = this.get("value");
					var valid = this.inherited("validator", arguments);
					var originalValue = t.objectCopy.get("dischargeDate");
					var valueChanged = (value && originalValue && value.getTime() != originalValue.getTime());

					if (value && (!originalValue || valueChanged) && valid) {
						var today = new Date();
						today.setHours(0, 0, 0, 0);
						if (!patientInfo.trainingPatient) {
							var minDate = date.add(today, "day", -parseInt(settings.dischargeDaysPast));
							var maxDate = date.add(today, "day", parseInt(settings.dischargeDaysFuture));

							var admitDate = locale.parse(patientInfo.admitDate, { selector: "date", datePattern: "yyyy-MM-dd" });
							var socDate = t.object.get("socDate");
							minDate = new Date(Math.max.apply(null, [minDate, admitDate, socDate]));

							if (value < minDate) {
								this.set("invalidMessage", "Date cannot be prior to " + locale.format(minDate, { selector: 'date' }));
								valid = false;
							} else if (value > maxDate) {
								this.set("invalidMessage", "Date cannot be after " + locale.format(maxDate, { selector: 'date' }));
								valid = false;
							}
						}
					} else {
						this.set("invalidMessage", t.invalidDateText);
					}

					return valid;
				};
			}

			if (t.transferDateField) {
				t.transferDateField.validator = function() {
					var value = this.get("value");
					var valid = this.inherited("validator", arguments);
					var originalValue = t.objectCopy.get("transferDate");
					var valueChanged = (value && originalValue && value.getTime() != originalValue.getTime());

					if (value && (!originalValue || valueChanged) && valid) {
						var today = new Date();
						today.setHours(0, 0, 0, 0);
						
						var admitDate = locale.parse(patientInfo.admitDate, { selector: "date", datePattern: "yyyy-MM-dd" });
						var socDate = t.object.get("socDate");
						var dischargeDate = t.getValidationValue("dischargeDate");
						var rocDate = t.getValidationValue("rocDate");

						if (!patientInfo.switchoverPatient && admitDate && value < admitDate) {
							this.set("invalidMessage", "Transfer Date cannot be before the admit date.");
							valid = false;
						} else if (!patientInfo.switchoverPatient && socDate && value < socDate) {
							this.set("invalidMessage", "Transfer Date cannot be before the SOC date.");
							valid = false;
						} else if (patientInfo.patientStatus === 7 && value > dischargeDate) {
							this.set("invalidMessage", "Transfer Date cannot be after the Agency Discharge date.");
							valid = false;
						} else if ((!patientInfo.trainingPatient) && (value > today)) {
							this.set("invalidMessage", "Transfer Date cannot be a future date.");
							valid = false;
						} else if (t.actionValue === "Modify" && rocDate && value > rocDate) {
							this.set("invalidMessage", "Transfer Date cannot be after the ROC date.");
							valid = false;
						} else if (t.actionValue === "Transfer" && rocDate && value < rocDate) {
							this.set("invalidMessage", "Transfer Date cannot be prior to the last ROC date.");
							valid = false;
						}
					} else {
						this.set("invalidMessage", t.invalidDateText);
					}

					return valid;
				};
			}

			if (t.rocDateField) {
				t.rocDateField.validator = function() {
					var value = this.get("value");
					var valid = this.inherited("validator", arguments);
					var originalValue = t.objectCopy.get("rocDate");
					var valueChanged = (value && originalValue && value.getTime() != originalValue.getTime());

					if (value && (!originalValue || valueChanged) && valid) {
						var transferDate = t.getValidationValue("transferDate");
						if (!transferDate) {
							transferDate = locale.parse(patientInfo.transferDate, { selector: "date", datePattern: "yyyy-MM-dd" });
						}

						if (transferDate && value < transferDate) {
							this.set("invalidMessage", "ROC Date cannot be prior to the transfer date.");
							valid = false;
						}
					} else {
						this.set("invalidMessage", t.invalidDateText);
					}

					return valid;
				};
			}

			if (t.snaDateField) {
				t.snaDateField.validator = function() {
					var value = this.get("value");
					var valid = this.inherited("validator", arguments);
					var originalValue = t.objectCopy.get("snaDate");
					var valueChanged = (value && originalValue && value.getTime() != originalValue.getTime());

					if (value && (!originalValue || valueChanged) && valid) {
						if (!patientInfo.documentationOnPaper && patientInfo.firstVisitDate) {
							var firstVisitDate = locale.parse(patientInfo.firstVisitDate, { selector: "date", datePattern: "yyyy-MM-dd" });

							if (value < firstVisitDate) {
								this.set("invalidMessage", "SNA date cannot be prior to the first visit date.");
								valid = false;
							}
						}
					} else {
						this.set("invalidMessage", t.invalidDateText);
					}

					return valid;
				};
			}

			if (t.anticipatedFirstVisitField) {
				t.anticipatedFirstVisitField.validator = function() {
					var value = this.get("value");
					var valid = this.inherited("validator", arguments);
					var originalValue = t.objectCopy.get("anticipatedFirstVisit");
					var valueChanged = (value && originalValue && value.getTime() != originalValue.getTime());

					if (value && (!originalValue || valueChanged) && valid) {
						var validation = anticipatedFirstVisitValidation(value);
						if (!validation.valid) {
							this.set("invalidMessage", validation.message);
							valid = false;
						}
					} else {
						this.set("invalidMessage", t.invalidDateText);
					}

					return valid;
				};
			}

			var anticipatedFirstVisitValidation = function(value) {
				var rocStatus = (t.actionValue = "ROC");
				var trfStatus = (t.actionValue = "Transfer");

				var transferDate = t.getValidationValue("transferDate");
				var rocDate = t.getValidationValue("rocDate");

				if (t.actionValue != "Transfer" && !transferDate) {
					transferDate = locale.parse(patientInfo.transferDate, { selector: "date", datePattern: "yyyy-MM-dd" });
				}

				if (t.actionValue != "ROC" && !rocDate) {
					rocDate = locale.parse(patientInfo.rocDate, { selector: "date", datePattern: "yyyy-MM-dd" });
				}

				if (rocDate && rocDate > transferDate) {
					rocStatus = true;
				}

				if (transferDate && transferDate > rocDate) {
					trfStatus = true;
				}

				var message = null;
				var valid = true;
				if (rocStatus && rocDate) {
					if (value < rocDate) {
						if (t.actionValue !== "ROC") {
							message = "WARNING: The anticipated 1st visit date is prior to the ROC date.";
						} else {
							message = "The anticipated 1st visit date cannot be prior to the ROC date.";
							valid = false;
						}
					}
				} else if (trfStatus && transferDate) {
					if (value < transferDate) {
						if (t.actionValue !== "Transfer") {
							message = "WARNING: The anticipated 1st visit date is prior to the Transfer date.";
						} else {
							message = "The anticipated 1st visit date cannot be prior to the Transfer date.";
							valid = false;
						}
					}
				}
				return { valid: valid, message: message };
			};
		},

		// For validation, use the value shown on screen when possible to avoid confusion
		getValidationValue: function(field) {
			var t = this;
			if (t[field + "Field"]) {
				return t[field + "Field"].get("value");
			} else {
				return t.object.get("value");
			}
		},

		_postModify: function() {
			var t = this;
			if (!t.inObjectLoad) {
				t.inherited(arguments);

				if (t.locationOfCareWidget) {
					t.locationOfCareWidget.updateStartDateConstraints();
					t.locationOfCareWidget.validateLocationOfCareItems();
				}
				if (t.locWidget) {
					t.locWidget.updateLevelStartDateConstraints();
					t.locWidget.validateLevelsOfCare();
				}

				// Use this instead of t.formNode.validate() to avoid focusing on first invalid widget
				arrayUtils.map(t.formNode._getDescendantFormWidgets(), function(widget) {
					if (!widget.disabled && widget.validate) {
						widget.validate();
					}
				});
			}
		},

		okToSave: function() {
			var t = this;
			var ok = false;

			if (!t.formNode.validate() || (t.locWidget && !t.locWidget.validate()) || (t.locationOfCareWidget && !t.locationOfCareWidget.validate())) {
				actionShowMessage("Please correct any invalid fields.");
			} else if (query(".pinkBackground", t.domNode).length > 0) {
				actionShowMessage("Please fill in all required fields.");
			} else if (t.actionType === "Modify" && !t.get("modified")) {
				actionShowMessage("You have not made any changes.");
			} else {
				ok = true;
			}
			return ok;
		},

		save: function() {
			var t = this;
			var deferred = new Deferred();
			var promise = deferred.promise;

			if (t.okToSave()) {
				var editView = registry.byId("editView");
				editView.disableSaveButtons(true);

				var saveData = t.object.getData();

				if (t.locationOfCareWidget) {
					saveData.locationOfCareSaveString = t.locationOfCareWidget.get("value");
				}
				if (t.locWidget) {
					saveData.levelOfCareSaveString = t.locWidget.get("value");
				}

				var options = {
					ndocNumber: ndocNumber,
					action: editView.get("actionValue"),
					saveData: saveData,
					userId: userId,
					deviceAppId: deviceAppId,
					deviceId: deviceId
				};

				promise = serverRequest("Patient.Status.Utility", "Save", options, false, 15000).then(function() {
					editView.disableSaveButtons(false);
					registry.byId("contentView").refresh();
				}, function(error) {
					editView.disableSaveButtons(false);
					actionShowMessage(error);
					throw error;
				});
			} else {
				deferred.reject();
			}
			return promise;
		},

		actionValueLogic: function() {
			var t = this;
			actionValue = t.actionValue;

			if (actionValue === "Modify") {
				// MODCHK^WDREFDCaux

				t.disable("anticipatedFirstVisit", (patientInfo.patientStatus === 5 || patientInfo.patientStatus === 7));

				
				if (patientInfo.patientStatus === 7) {
					t.disable("dischargeDate", false);
					t.disable("dischargeReason", false);
					t.disable("location", false);
					t.disable("facility", false);
					t.disable("notificationDate", false);

					t.require("dischargeDate");
					t.require("dischargeReason");
				} else if (patientInfo.patientStatus === 5) {
					t.require("snaDate");
					t.require("snaReason");

					if (!settings.sna.allowErrorMeds) {
						t.disable("errorMedications");
						domStyle.set(t.errorMedicationsContainer, "display", "none");
					}

					if (!settings.sna.allowErrorOrders) {
						t.disable("errorOrders");
						domStyle.set(t.errorOrdersContainer, "display", "none");
					}
				} else if (patientInfo.patientStatus === 4 && patientInfo.rnsReason) {
					t.require("rnsDate");
					t.require("rnsReason");

					if (!settings.rns.allowErrorMeds) {
						t.disable("errorMedications");
						domStyle.set(t.errorMedicationsContainer, "display", "none");
					}

					if (!settings.rns.allowErrorOrders) {
						t.disable("errorOrders");
						domStyle.set(t.errorOrdersContainer, "display", "none");
					}
				}


				var disableTransferReason = true;
				var disableTransferDate = true;
				if (patientInfo.patientStatus !== 7 && t.object.get("transferDate")) {
					//disableFacilityLocation = false;
					disableTransferReason = false;
					disableTransferDate = false;
				}

				t.disable("rocDate", !t.object.get("rocDate"));										// Enable if it already has a value
				t.disable("transferDate", (!t.object.get("transferDate") && disableTransferDate));
				t.disable("transferReason", disableTransferReason);

				//t.disable("facility", disableFacilityLocation);
				//t.disable("location", disableFacilityLocation);

				if (patientType === "Hospice") {
					t.disable("agencyAdmitReason", !patientInfo.socDate);
					t.require("agencySocDate", (patientInfo.patientStatus == 6));
					t.require("agencyAdmitReason", (patientInfo.patientStatus == 6));
					t.require("admittedFrom", (patientInfo.patientStatus == 6));
					t.locWidget && t.locWidget.set("disabled", false);
					t.disable("startingCert", false);
				}
				
				t.locationOfCareWidget && t.locationOfCareWidget.set("disabled", false);

				if (patientInfo.patientStatus == 7 || patientInfo.patientStatus == 5 || (patientInfo.patientStatus === 4 && patientInfo.rnsReason !== "")) {
					var date;
					switch(patientInfo.patientStatus) {
						case 7:
							date = patientInfo.deathDate;
							break;
						case 5:
							date = patientInfo.snaDate;
							break;
						case 4:
							date = patientInfo.rnsDate;
					}
					serverRequest("Patient.Status.Utility", "FutureActivitiesCheck", {ndocNumber: ndocNumber, date: date}).then(function(results) {
						t.disable("deleteScheduledMeetings", !results.scheduledMeetings);
					});
				} else {
					t.disable("deleteScheduledMeetings");
				}
			} else if (actionValue === "Admit To Hospice") {
				// ADMHOSPCHK^WDREFDCaux
				t.require("hospiceBenefitDate");
				t.require("agencySocDate");
				t.require("agencyAdmitReason");
				t.require("startingCert");
				t.require("admittedFrom");
				t.disable("agencyAdmitReason", !patientInfo.socDate);
				t.locWidget && t.locWidget.actionAdd();
			} else if (actionValue === "RNS") {
				// RNSCHK^WDREFDCaux
				t.require("rnsDate");
				t.require("rnsReason");

				if (settings.rns.allowErrorMeds) {
					t.require("errorMedications");
				} else {
					t.disable("errorMedications");
					domStyle.set(t.errorMedicationsContainer, "display", "none");
				}

				if (settings.rns.allowErrorOrders) {
					t.require("errorOrders");
				} else {
					t.disable("errorOrders");
					domStyle.set(t.errorOrdersContainer, "display", "none");
				}

				if (settings.employeeActivitySite) {
					t.require("deleteScheduledVisits");
					t.require("deleteScheduledMeetings");
				}
			} else if (actionValue === "SNA") {
				// SNACHK^WDREFDCaux
				t.require("snaDate");
				t.require("snaReason");

				if (settings.sna.allowErrorMeds) {
					t.require("errorMedications");
				} else {
					t.disable("errorMedications");
					domStyle.set(t.errorMedicationsContainer, "display", "none");
				}

				if (settings.sna.allowErrorOrders) {
					t.require("errorOrders");
				} else {
					t.disable("errorOrders");
					domStyle.set(t.errorOrdersContainer, "display", "none");
				}

				if (settings.employeeActivitySite) {
					t.require("deleteScheduledVisits");
					t.require("deleteScheduledMeetings");
				}
			} else if (actionValue === "Transfer") {
				// TRFCHK^WDREFDCaux

				t.require("transferDate");
				t.require("transferReason");
				t.require("facility", settings.transfer.facilityReq);
				t.require("location", settings.transfer.locationReq);

				if (settings.transfer.allowDiscontinueMeds) {
					t.require("discontinueMedications");
				} else {
					t.disable("discontinueMedications");
					domStyle.set(t.discontinueMedicationsContainer, "display", "none");
				}

				if (settings.transfer.allowDeleteIncompleteMeds) {
					t.require("deleteIncompleteMedications", patientInfo.hasIncompleteMeds);
					t.disable("deleteIncompleteMedications", !patientInfo.hasIncompleteMeds);
				} else {
					t.disable("deleteIncompleteMedications");
					domStyle.set(t.deleteIncompleteMedicationsContainer, "display", "none");
				}

				if (settings.transfer.allowHoldMeds) {
					t.require("holdActiveMedications");
				} else {
					t.disable("holdActiveMedications");
					domStyle.set(t.holdActiveMedicationsContainer, "display", "none");
				}

				if (settings.transfer.allowDiscontinueOrders) {
					t.require("discontinueOrders");
				} else {
					t.disable("discontinueOrders");
					domStyle.set(t.discontinueOrdersContainer, "display", "none");
				}

				if (settings.transfer.allowDeleteIncompleteOrders) {
					t.require("deleteIncompleteOrders");
					t.require("deleteIncompleteOrders", patientInfo.hasIncompleteMeds);
					t.disable("deleteIncompleteOrders", !patientInfo.hasIncompleteMeds);
				} else {
					t.disable("deleteIncompleteOrders");
					domStyle.set(t.deleteIncompleteOrdersContainer, "display", "none");
				}

				if (settings.transfer.allowHoldOrders) {
					t.require("holdActiveOrders");
				} else {
					t.disable("holdActiveOrders");
					domStyle.set(t.holdActiveOrdersContainer, "display", "none");
				}
			} else if (actionValue === "ROC") {
				// ROCCHK^WDREFDCaux
				t.require("rocDate");

				if (settings.roc.allowResumeMeds) {
					t.require("resumeMeds", patientInfo.hasTransferCreatedHoldMeds);
					t.disable("resumeMeds", !patientInfo.hasTransferCreatedHoldMeds);
				} else {
					t.disable("resumeMeds");
					domStyle.set(t.resumeMedsContainer, "display", "none");
				}

				if (settings.roc.allowResumeOrders) {
					t.require("resumeOrders", patientInfo.hasTransferCreatedHoldOrders);
					t.disable("resumeOrders", !patientInfo.hasTransferCreatedHoldOrders);
				} else {
					t.disable("resumeOrders");
					domStyle.set(t.resumeOrdersContainer, "display", "none");
				}
			} else if (actionValue === "Cancel Patient") {
				domStyle.set(t.confirmCancelPatientMatchLabelContainer, "width", "auto");
				domStyle.set(t.cancelPatientWarningLabelContainer, "width", "auto");				
				t.disable("referralDate");
				t.disable("socDate");
				t.disable("dischargeDate");
			} else if (actionValue === "Discharge") {
				// DCPCHK^WDREFDCaux

				t.require("dischargeDate");
				t.require("dischargeReason");

				if (settings.discharge.allowDiscontinueMeds) {
					t.require("discontinueMedications");
				} else {
					t.disable("discontinueMedications");
					domStyle.set(t.discontinueMedicationsContainer, "display", "none");
				}

				if (settings.discharge.allowDeleteIncompleteMeds) {
					t.require("deleteIncompleteMedications", patientInfo.hasIncompleteMeds);
					t.disable("deleteIncompleteMedications", !patientInfo.hasIncompleteMeds);
				} else {
					t.disable("deleteIncompleteMedications");
					domStyle.set(t.deleteIncompleteMedicationsContainer, "display", "none");
				}

				if (settings.discharge.allowDiscontinueOrders) {
					t.require("discontinueOrders");
				} else {
					t.disable("discontinueOrders");
					domStyle.set(t.discontinueOrdersContainer, "display", "none");
				}

				if (settings.discharge.allowDeleteIncompleteOrders) {
					t.require("deleteIncompleteOrders", patientInfo.hasIncompleteOrders);
					t.disable("deleteIncompleteOrders", !patientInfo.hasIncompleteOrders);
				} else {
					t.disable("deleteIncompleteOrders");
					domStyle.set(t.deleteIncompleteOrdersContainer, "display", "none");
				}

				t.require("facility", settings.discharge.facilityReq);
				t.require("location", settings.discharge.locationReq);

				if (settings.discharge.allowPhysicianOrder) {
					t.require("addDischargeOrder");
				} else {
					t.disable("addDischargeOrder", true);
					domStyle.set(t.addDischargeOrderContainer, "display", "none");
					domStyle.set(t.dischargeOrderPhysicianContainer, "display", "none");
					domStyle.set(t.dischargeOrderDisciplineContainer, "display", "none");
				}
			} else if (actionValue === "Unfile Referral") {

			} else if (actionValue === "Cancel Admit") {
				// CANADMCHK^WDREFDCaux
				if (patientInfo.nonCanceledChartedVisits == 1) {
					t.disable("rnsDate");
					t.disable("rnsReason");
					t.require("snaDate");
					t.require("snaReason");
				} else if (patientInfo.nonCanceledChartedVisits == 0) {
					t.disable("snaDate");
					t.disable("snaReason");
					t.require("rnsDate");
					t.require("rnsReason");
				} else {
					// shouldn't happen
				}
			} 
			
			// DEPRECATED ND-9045
			/*else if (actionValue === "Cancel RNS") { // DEPRECATED ND-9045
				// CANRNSCHK^WDREFDCaux
			} else if (actionValue === "Cancel SNA") {
				// CANRNSCHK^WDREFDCaux
			}*/ 
			
			else if (actionValue === "Cancel Discharge") {
				// CANDCCHK^WDREFDCaux
			}

			if (!settings.employeeActivitySite) {
				if (t.deleteScheduledVisitsContainer) {
					domStyle.set(t.deleteScheduledVisitsContainer, "display", "none");
					t.disable("deleteScheduledVisits", true);
				}
				if (t.deleteScheduledMeetingsContainer) {
					domStyle.set(t.deleteScheduledMeetingsContainer, "display", "none");
					t.disable("deleteScheduledMeetings", true);
				}
			}
		},

		show: function() {
			var t = this;
			if (t.locationOfCareWidget) {
				// The Location Of Care grid doesn't display correctly without this
				t.locationOfCareWidget.grid.resize();
			}
			if (t.locWidget) {
				// The LOC grid doesn't display correctly without this
				t.locWidget.grid.resize();
			}
		}
	});

	return StatusEditor;
	}
);